
#include "defs.h"
#include "eb123.h"
#include "ebook.h"
#include "headwordrender.h"

G_DEFINE_TYPE(EbHeadwordRender, eb_headword_render, GTK_TYPE_CELL_RENDERER);

static void eb_headword_render_get_size(GtkCellRenderer *cell,
	GtkWidget *widget,
	GdkRectangle *area,
	gint *x_offset,
	gint *y_offset,
	gint *w,
	gint *h);

static void eb_headword_render(GtkCellRenderer      *cell,
	GdkWindow            *window,
	GtkWidget            *widget,
	GdkRectangle         *background_area,
	GdkRectangle         *cell_area,
	GdkRectangle         *expose_area,
	GtkCellRendererState  flags);


static gpointer eb_headword_render_parent_class_ptr = NULL;

enum
{
    HWR_PROP_0,
    HWR_PROP_TEXT,
    HWR_PROP_RESULT,
};

static void eb_headword_render_set_property(GObject *object, guint param_id, const GValue *value, GParamSpec *pspec)
{
    EbHeadwordRender *r = EB_HEADWORD_RENDER(object);

    switch(param_id)
    {
	case HWR_PROP_TEXT:
	    if(r->text)
		g_free(r->text);
	    r->text = g_strdup(g_value_get_string(value));
	    break;
	case HWR_PROP_RESULT:
	    r->res = ((RESULT*)g_value_get_pointer(value));
	    break;
	default:
	    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
	    break;
    }
}

static void eb_headword_render_get_property(GObject *object, guint param_id, GValue *value, GParamSpec *pspec)
{
    EbHeadwordRender *r = EB_HEADWORD_RENDER(object);

    switch (param_id)
    {
	case HWR_PROP_TEXT:
	    g_value_set_string(value, r->text);
	    break;
	case HWR_PROP_RESULT:
	    g_value_set_pointer(value, r->res);
	    break;
	default:
	    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
	    break;
    }
}

typedef struct _eb_headword_render_ctx
{
    guint x, y;
    gboolean sub, sup;
    gboolean pretend;
    GdkWindow *window;
    GtkWidget *widget;
    GtkCellRendererState flags;
    EbHeadwordRender *r;
} eb_headword_render_ctx;

static void eb_headword_render_characters(void *ctx, const xmlChar *ch, int len)
{
    eb_headword_render_ctx *my = (eb_headword_render_ctx*)ctx;
    gint x_offset = my->x, y_offset = my->y, cell_state = gtk_widget_get_state(my->widget);
    gchar *str, *tmp, *spec, *text;
    PangoAttrList *attrs;
    PangoLayout *layout;
    PangoRectangle rect;
    GtkStateType state = (my->flags & GTK_CELL_RENDERER_SELECTED) ? GTK_STATE_SELECTED : GTK_STATE_NORMAL;
    spec = gtk_color_selection_palette_to_string(&(my->widget->style->fg[state]), 1);
    if(my->sub)
	str = g_strdup_printf("<sub>%s</sub>", (gchar*)ch);
    else if(my->sup)
	str = g_strdup_printf("<sup>%s</sup>", (gchar*)ch);
    else
	str = g_strdup((gchar*)ch);
    tmp = str;
    str = g_strdup_printf("<span foreground=\"%s\">%s</span>", spec, tmp);
    g_free(tmp);
    pango_parse_markup(str, -1, 0, &attrs, &text, NULL, NULL);
    if(text)
    {
	layout = gtk_widget_create_pango_layout(my->widget, text);
	pango_layout_set_attributes(layout, attrs);
    }
    else
	layout = gtk_widget_create_pango_layout(my->widget, str);
    pango_layout_get_pixel_extents (layout, NULL, &rect);
    if(!my->pretend)
	gdk_draw_layout(my->window, my->widget->style->fg_gc[cell_state], x_offset, y_offset, layout);
    my->x += rect.width;
    if(str) g_free(str);
    if(spec) g_free(spec);
}

static void eb_headword_render_startElement(void *ctx, const xmlChar *name, const xmlChar **atts)
{
    eb_headword_render_ctx *my = (eb_headword_render_ctx*)ctx;
    if(!g_strcmp0((gchar*)name, "gaiji"))
    {
	gint w, h, x_offset, y_offset;;
	GtkStateType state = (my->flags & GTK_CELL_RENDERER_SELECTED) ? GTK_STATE_SELECTED : GTK_STATE_NORMAL;
	gchar *spec = gtk_color_selection_palette_to_string(&(my->widget->style->fg[state]), 1);
	GdkPixbuf *pixbuf;
	GdkColor color;
	gdk_color_parse(spec, &color);
	if(spec) g_free(spec);
	eb_headword_render_get_size(NULL, my->widget, NULL, &x_offset, &y_offset, NULL, NULL);
	if((pixbuf = ebook_read_gaiji(my->r->res->binfo, (gchar*)atts[1], &w, &h)))
	{
	    if(!my->pretend)
	    {
		GdkGC *gc = gdk_gc_new(my->window);
#if 0
		gdk_gc_set_foreground(gc, &color);
		gdk_gc_set_rgb_fg_color(gc, &color);
		gdk_gc_set_fill(gc, GDK_OPAQUE_STIPPLED);
#endif
		gdk_draw_pixbuf(my->window, gc, pixbuf, 0, 0, my->x, my->y, w, h + y_offset, GDK_RGB_DITHER_NORMAL, 0, 0);
		g_object_unref(gc);
	    }
	    my->x += w + x_offset;
	}
	gdk_pixbuf_unref(pixbuf);
    }
    if(!g_strcmp0((gchar*)name, "sub"))
	my->sub = TRUE;
    if(!g_strcmp0((gchar*)name, "sup"))
	my->sup = TRUE;
}

static void eb_headword_render_endElement(void *ctx, const xmlChar *name)
{
    eb_headword_render_ctx *my = (eb_headword_render_ctx*)ctx;
    if(!g_strcmp0((gchar*)name, "sub"))
	my->sub = FALSE;
    if(!g_strcmp0((gchar*)name, "sup"))
	my->sup = FALSE;
}

static void eb_headword_render_get_size(GtkCellRenderer *cell,
	GtkWidget *widget,
	GdkRectangle *area,
	gint *x_offset,
	gint *y_offset,
	gint *w,
	gint *h)
{
    EbHeadwordRender *r = (EbHeadwordRender*)cell;
    if(w)
	*w = 200;
    if(h)
    {
	if(!GTK_IS_WIDGET(widget))
	    return;
	GtkStyle *style = gtk_widget_get_style(widget);
	*h = 2*pango_font_description_get_size(style->font_desc)/PANGO_SCALE;
    }
    if(x_offset)
	*x_offset = 2;
    if(y_offset)
	*y_offset = 2;
    if(r)
    {
	eb_headword_render(cell, NULL, widget, NULL, area, NULL, 0);
	*w = r->width;
    }
}

/*
 *  Workaround: slightly modify brackets
 *  in heading text for proper parsing by libxml, e.g.
 *  日本語 <eng> -> 日本語 [eng]
 */

static gchar* eb_headword_find_br2ptr(gchar *br)
{
    gint c = 0;
    while(br[0])
    {
	if((br[0] == '>') && !c)
	    return br;
	if(br[0] == '<') c++;
	if(br[0] == '>') c--;
	br++;
    }
    g_assert(0);
    return NULL;
}

static void eb_headword_fix_text(gchar *txt)
{
    gint len = strlen(txt);
    gchar *br = txt;
    while((br = g_strstr_len(br, len - (br - txt), "<")))
    {
	if((br[1] == '/') || \
	    !g_ascii_strncasecmp(br + 1, "sub", 3) || \
	    !g_ascii_strncasecmp(br + 1, "sup", 3) || \
	    !g_ascii_strncasecmp(br + 1, "gaiji", 5))
	{
	    br++;
	    continue;
	}
	br[0] = '[';
	gchar *br2 = eb_headword_find_br2ptr(br);
	if(br2) br2[0] = ']';
    }
}

static void eb_headword_render(GtkCellRenderer      *cell,
	 GdkWindow            *window,
	 GtkWidget            *widget,
	 GdkRectangle         *background_area,
	 GdkRectangle         *cell_area,
	 GdkRectangle         *expose_area,
	 GtkCellRendererState  flags)
{
    EbHeadwordRender *r = EB_HEADWORD_RENDER(cell);
    eb_headword_render_ctx ctx;
    ctx.pretend = !(window || background_area || expose_area || flags);
    ctx.x = ctx.pretend ? 0 : cell_area->x;
    ctx.y = ctx.pretend ? 0 : cell_area->y;
    ctx.sub = ctx.sup = FALSE;
    ctx.window = window;
    ctx.widget = widget;
    ctx.flags = flags;
    ctx.r = EB_HEADWORD_RENDER(cell);
    htmlSAXHandler cb;
    memset(&cb, 0, sizeof(htmlSAXHandler));
    cb.startElement = &eb_headword_render_startElement;
    cb.endElement = &eb_headword_render_endElement;
    cb.characters = &eb_headword_render_characters;
    eb_headword_fix_text(r->text);
    htmlDocPtr doc = htmlSAXParseDoc((xmlChar*)r->text, ENC_UTF8, &cb, &ctx);
    r->width = ctx.x;
    xmlFreeDoc(doc);
}

static void eb_headword_render_finalize(GObject *object)
{
    EbHeadwordRender *r = EB_HEADWORD_RENDER(object);
    if(r->text)
	g_free(r->text);
    (*G_OBJECT_CLASS(eb_headword_render_parent_class_ptr)->finalize)(object);
}

static void eb_headword_render_class_init(EbHeadwordRenderClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
    GtkCellRendererClass *eklass = GTK_CELL_RENDERER_CLASS(klass);
    eb_headword_render_parent_class_ptr = g_type_class_peek_parent(klass);
    gobject_class->get_property = eb_headword_render_get_property;
    gobject_class->set_property = eb_headword_render_set_property;
    eklass->get_size = eb_headword_render_get_size;
    eklass->render = eb_headword_render;
    gobject_class->finalize = eb_headword_render_finalize;
    g_object_class_install_property(gobject_class, HWR_PROP_TEXT, g_param_spec_string("text", _("Text"), _("Text to render"), NULL, G_PARAM_READWRITE));
    g_object_class_install_property(gobject_class, HWR_PROP_RESULT, g_param_spec_pointer("result", _("Result"), _("Search result"), G_PARAM_READWRITE));
}

static void eb_headword_render_init(EbHeadwordRender *self)
{
    self->text = NULL;
    self->res = NULL;
    self->width = 100;
    GTK_CELL_RENDERER(self)->xalign = 0.0;
    GTK_CELL_RENDERER(self)->yalign = 0.5;
    GTK_CELL_RENDERER(self)->xpad = 2;
    GTK_CELL_RENDERER(self)->ypad = 2;
}

